(document:surround "/std/frame")
(document:insert "/std/functions")

(define wizardface (make-widget "wizardface"))

;;; Functions
(define *step* (make-cell 0))
(define *step-list* (make-cell '()))

(define (step) (cell-ref *step*))
(define (++step) (cell-set! *step* (+ (step) 1)) (step))
(define (--step) (cell-set! *step* (- (step) 1)) (step))
(define (last) (- (length (cell-ref *step-list*)) 1))

(define (url) (car (list-ref (cell-ref *step-list*) (step))))
(define (topic) (cdr (list-ref (cell-ref *step-list*) (step))))

(define (popup-overflow proc cb)
  (document:popup-critical
   (string-append "Please remove " proc " call from " cb " function"))
   'cancel)

(define (sure-yes key name list)
  (or (member key list)
      (wizard-id action-add (vector key name))))

(define (sure-no key name list)
  (and (member key list)
       (wizard-id action-remove key)))

(define (action-name x) (vector-ref x 0))

(define (check-buttons step last)
  (with-translation _ "alterator-wizard"
                    (let ((buttons (map action-name (wizard-id actions))))
                      (cond
                       ((= step 0)
                        (sure-yes 'forward (_ "Next") buttons)
                        (sure-no 'backward (_ "Previous") buttons)
                        (sure-no 'finish (_ "Finish") buttons))
                       ((< 0 step last)
                        (sure-yes 'forward (_ "Next") buttons)
                        (sure-yes 'backward (_ "Previous") buttons)
                        (sure-no 'finish (_ "Finish") buttons))
                       ((= step last)
                        (sure-no 'forward (_ "Next") buttons)
                        (sure-yes 'backward (_ "Previous") buttons)
                        (sure-yes 'finish (_ "Finish") buttons))))
                    #t))

(define (wizard:replace direction)
  (check-buttons (step) (last))
  (frame:replace (url) 'frame:direction direction)
  (document:root
    help (woo-get-option (woo-read-first "/help/" 'topic (topic)) 'url)))

(define (go-next-callback)
  (frame:set-next (thunk (popup-overflow "frame:next" "frame:on-next")))
  (begin-1
    (not (eq? (frame:call-on-next) 'cancel))
    (frame:set-next go-next)))

(define (go-next)
  (and (< (step) (last))
       (go-next-callback)
       (wizard-id current-step (++step))
       (wizard:replace 'next)))

(define (go-back-callback)
  (frame:set-back (thunk (popup-overflow "frame:back" "frame:on-back")))
  (begin-1
    (not (eq? (frame:call-on-back) 'cancel))
    (frame:set-back go-back)))

(define (go-back)
  (and (> (step) 0)
       (go-back-callback)
       (wizard-id current-step (--step))
       (wizard:replace 'back)))

(define (go-end)
  (frame:set-end (thunk (popup-overflow "frame:end" "frame:on-next")))
  (or (eq? 'cancel (frame:call-on-next))
      (document:end))
  (frame:set-end go-end))

(define (name+help x)
  (cons (woo-get-option x 'name)
        (woo-get-option x 'help)))

(define (label+icon x)
  (cons (woo-get-option x 'label)
        (woo-get-option x 'icon)))

;;; UI
(frame:init-on-next)
(frame:init-on-back)

(frame:set-next go-next)
(frame:set-back go-back)
(frame:set-end go-end)

(document:id wizard-id
  (frame:set-wizard
    (wizardface
      title "Installer"
      (frame:page)
      (when clicked
        (case (wizard-id current-action)
          ((finish) (frame:end))
          ((forward) (frame:next))
          ((backward) (frame:back)))))))

;;; Logic
(document:root
 (when loaded
   (let ((step-list (woo-list "/step-list")))
     (cell-set! *step-list* (map name+help step-list))
     (wizard-id steps (map label+icon step-list)
                current-step 0)
     (wizard-id action-add 'help (_ "Help"))
     (wizard:replace 'next))))
